/* Copyright (C) 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package backend.srb;

import osio.TextPrinter;
import backend.LittleEInstr;

/**
 * SRBInstruction: instruction- and block-super type 
 * 
 * @author S. Frenz
 * @version 090214 got parent from SRBI_Container, added phase to convertToNative
 *  version 090207 added copyright notice
 *  version 090118 added enumerateInstructions and print
 *  version 081226 initial version
 */

public abstract class SRBInstruction extends LittleEInstr {
  public SRB owner;
  public SRBI_Container outer;
  
  public SRBInstruction(SRB io, int maxInstrSize) {
    super(maxInstrSize);
    owner=io;
  }
  public abstract void resetMark(); //reset isDest
  public abstract boolean isMarked(); //returns true if at least one instruction is marked
  public abstract void generateJumpDestsForTarget(); //generate unlinked target instructions as target jump destinations
  public abstract void convertToNative(int phase); //treat current instruction regarding current phase
  public abstract void cleanUp(); //removes next-listing!
  public abstract int enumerateInstructions(int startWith); //enumerate instruction(s) and return next number
  public abstract int print(TextPrinter v); //print instruction(s) and return instruction cound
}
